#include <sstream>
#include <cmath>
#include "common.h"

using namespace std;

namespace cmmn {

	long double p, pa;

	int N, k;

	long double * LogCombi;



	//     Bisection

	void Bisection( long double (*f)(long double),
			  long double s,			//  f is increasing -> s > 0.0
								//  f is decreasing -> s < 0.0
			  long double c,			//  f(Root) = c
			  long double L_b, 			//  L_b < Root < U_b
			  long double U_b,		
			  long double & Root,		//  Set Root so that f(Root) = c
			  long double acc ){			//  accuracy
		if ( (f(L_b) - c) * (f(U_b) - c) > 0.0 ){
			ostringstream sout;
			sout << "Invalid values...L_b = " << L_b << "  U_b = " << U_b;
			string strmsg = sout.str();
			char * cstr = new char[strmsg.length() + 1];
			for (int i = 0; i < strmsg.length(); i++)
				cstr[i] = strmsg[i];
			cstr[strmsg.length()] = 0;

			throw cstr;
		}
		do {
			long double m = 0.5 * (L_b + U_b);
			long double v = f(m);
			if ( s * (v - c) > 0.0 ) {
				U_b = m;
			} else {
				L_b = m;
			}
		} while ( (abs(U_b - L_b) >= acc * abs(U_b))
					&&
				  ( (abs(U_b) + abs(L_b)) >= 1.0e-14)
				);
		Root = 0.5 * (L_b + U_b);
	}



	long double prob_term( long double p, int N, int i ){
		if (p <= 0.0){
				if (i >= 1) return 0.0;
				else		return 1.0;
		}else if (p >= 1.0){
				if (i == N) return 1.0;
				else		return 0.0;
		}else{
			return exp(LogCombi[i] + (i * log(p) + (N - i) * log(1.0 - p)));
		}
	}

	long double Cum_prob( long double p ){
		long double v = 0.0;
		for (int i = 0; i <= k; i++)
			v += prob_term( p, N, i );
		return v;
	}


	long double UCum_prob( long double p ){
		long double v = 0.0;
		for (int i = N; i >= k; i--)
			v += prob_term( p, N, i );
		return v;
	}


}
